@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -20,19 +20,41 @@
 hypothesis[cli]
 ----------------
 
-This module provides Hypothesis' command-line interface, for e.g.
-:doc:`'ghostwriting' tests <ghostwriter>` via the terminal.
-It requires the :pypi:`click` package.
+::
 
-Run :command:`hypothesis --help` in your terminal for details.
+    $ hypothesis --help
+    Usage: hypothesis [OPTIONS] COMMAND [ARGS]...
+
+    Options:
+      --version   Show the version and exit.
+      -h, --help  Show this message and exit.
+
+    Commands:
+      codemod  `hypothesis codemod` refactors deprecated or inefficient code.
+      fuzz     [hypofuzz] runs tests with an adaptive coverage-guided fuzzer.
+      write    `hypothesis write` writes property-based tests for you!
+
+This module requires the :pypi:`click` package, and provides Hypothesis' command-line
+interface, for e.g. :doc:`'ghostwriting' tests <ghostwriter>` via the terminal.
+It's also where `HypoFuzz <https://hypofuzz.com/>`__ adds the :command:`hypothesis fuzz`
+command (`learn more about that here <https://hypofuzz.com/docs/quickstart.html>`__).
 """
 
 import builtins
 import importlib
 import sys
 from difflib import get_close_matches
+from functools import partial
+from multiprocessing import Pool
 
-from hypothesis.extra import ghostwriter
+MESSAGE = """
+The Hypothesis command-line interface requires the `{}` package,
+which you do not have installed.  Run:
+
+    python -m pip install --upgrade hypothesis[cli]
+
+and try again.
+"""
 
 try:
     import click
@@ -40,16 +62,7 @@ except ImportError:
 
     def main():
         """If `click` is not installed, tell the user to install it then exit."""
-        sys.stderr.write(
-            """
-The Hypothesis command-line interface requires the `click` package,
-which you do not have installed.  Run:
-
-    python -m pip install --upgrade hypothesis[cli]
-
-and try again.
-"""
-        )
+        sys.stderr.write(MESSAGE.format("click"))
         sys.exit(1)
 
 
@@ -67,6 +80,7 @@ else:
 
     def obj_name(s: str) -> object:
         """This "type" imports whatever object is named by a dotted string."""
+        s = s.strip()
         try:
             return importlib.import_module(s)
         except ImportError:
@@ -93,12 +107,84 @@ else:
                 + (f"  Closest matches: {matches!r}" if matches else "")
             )
 
+    def _refactor(func, fname):
+        try:
+            with open(fname) as f:
+                oldcode = f.read()
+        except (OSError, UnicodeError) as err:
+            # Permissions or encoding issue, or file deleted, etc.
+            return f"skipping {fname!r} due to {err}"
+        newcode = func(oldcode)
+        if newcode != oldcode:
+            with open(fname, mode="w") as f:
+                f.write(newcode)
+
+    @main.command()  # type: ignore  # Click adds the .command attribute
+    @click.argument("path", type=str, required=True, nargs=-1)
+    def codemod(path):
+        """`hypothesis codemod` refactors deprecated or inefficient code.
+
+        It adapts `python -m libcst.tool`, removing many features and config options
+        which are rarely relevant for this purpose.  If you need more control, we
+        encourage you to use the libcst CLI directly; if not this one is easier.
+
+        PATH is the file(s) or directories of files to format in place, or
+        "-" to read from stdin and write to stdout.
+        """
+        try:
+            from libcst.codemod import gather_files
+
+            from hypothesis.extra import codemods
+        except ImportError:
+            sys.stderr.write(
+                "You are missing required dependencies for this option.  Run:\n\n"
+                "    python -m pip install --upgrade hypothesis[codemods]\n\n"
+                "and try again."
+            )
+            sys.exit(1)
+
+        # Special case for stdin/stdout usage
+        if "-" in path:
+            if len(path) > 1:
+                raise Exception(
+                    "Cannot specify multiple paths when reading from stdin!"
+                )
+            print("Codemodding from stdin", file=sys.stderr)
+            print(codemods.refactor(sys.stdin.read()))
+            return 0
+
+        # Find all the files to refactor, and then codemod them
+        files = gather_files(path)
+        errors = set()
+        if len(files) <= 1:
+            errors.add(_refactor(codemods.refactor, *files))
+        else:
+            with Pool() as pool:
+                for msg in pool.imap_unordered(
+                    partial(_refactor, codemods.refactor), files
+                ):
+                    errors.add(msg)
+        errors.discard(None)
+        for msg in errors:
+            print(msg, file=sys.stderr)
+        return 1 if errors else 0
+
     @main.command()  # type: ignore  # Click adds the .command attribute
     @click.argument("func", type=obj_name, required=True, nargs=-1)
+    @click.option(
+        "--roundtrip",
+        "writer",
+        flag_value="roundtrip",
+        help="start by testing write/read or encode/decode!",
+    )
+    @click.option(
+        "--equivalent",
+        "writer",
+        flag_value="equivalent",
+        help="very useful when optimising or refactoring code",
+    )
     @click.option("--idempotent", "writer", flag_value="idempotent")
     @click.option("--binary-op", "writer", flag_value="binary_operation")
-    @click.option("--equivalent", "writer", flag_value="equivalent")
-    @click.option("--roundtrip", "writer", flag_value="roundtrip")
     # Note: we deliberately omit a --ufunc flag, because the magic()
     # detection of ufuncs is both precise and complete.
     @click.option(
@@ -123,11 +209,12 @@ else:
 
         \b
             hypothesis write gzip
+            hypothesis write numpy.matmul
             hypothesis write re.compile --except re.error
-            hypothesis write --style=unittest --idempotent sorted
-            hypothesis write --binary-op operator.add
             hypothesis write --equivalent ast.literal_eval eval
             hypothesis write --roundtrip json.dumps json.loads
+            hypothesis write --style=unittest --idempotent sorted
+            hypothesis write --binary-op operator.add
         """
         # NOTE: if you want to call this function from Python, look instead at the
         # ``hypothesis.extra.ghostwriter`` module.  Click-decorated functions have
@@ -141,4 +228,10 @@ else:
         elif writer == "equivalent" and len(func) == 1:
             writer = "fuzz"
 
+        try:
+            from hypothesis.extra import ghostwriter
+        except ImportError:
+            sys.stderr.write(MESSAGE.format("black"))
+            sys.exit(1)
+
         print(getattr(ghostwriter, writer)(*func, except_=except_ or (), style=style))
