@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -23,7 +23,6 @@ from hypothesis.errors import Frozen, InvalidArgument, StopTest
 from hypothesis.internal.compat import bit_length, int_from_bytes, int_to_bytes
 from hypothesis.internal.conjecture.junkdrawer import IntList, uniform
 from hypothesis.internal.conjecture.utils import calc_label_from_name
-from hypothesis.internal.escalation import mark_for_escalation
 
 TOP_LABEL = calc_label_from_name("top")
 DRAW_BYTES_LABEL = calc_label_from_name("draw_bytes() in ConjectureData")
@@ -34,8 +33,8 @@ class ExtraInformation:
     be added to the final ``ConjectureResult``."""
 
     def __repr__(self):
-        return "ExtraInformation(%s)" % (
-            ", ".join(["%s=%r" % (k, v) for k, v in self.__dict__.items()]),
+        return "ExtraInformation({})".format(
+            ", ".join([f"{k}={v!r}" for k, v in self.__dict__.items()]),
         )
 
     def has_information(self):
@@ -49,7 +48,7 @@ class Status(IntEnum):
     INTERESTING = 3
 
     def __repr__(self):
-        return "Status.%s" % (self.name,)
+        return f"Status.{self.name}"
 
 
 @attr.s(frozen=True, slots=True)
@@ -118,7 +117,7 @@ class Example:
         return (self.owner is not other.owner) or (self.index != other.index)
 
     def __repr__(self):
-        return "examples[%d]" % (self.index,)
+        return f"examples[{self.index}]"
 
     @property
     def label(self):
@@ -451,7 +450,7 @@ class Examples:
         assert isinstance(i, int)
         n = len(self)
         if i < -n or i >= n:
-            raise IndexError("Index %d out of range [-%d, %d)" % (i, n, n))
+            raise IndexError(f"Index {i} out of range [-{n}, {n})")
         if i < 0:
             i += n
         return Example(self, i)
@@ -583,7 +582,7 @@ class Blocks:
     def _check_index(self, i):
         n = len(self)
         if i < -n or i >= n:
-            raise IndexError("Index %d out of range [-%d, %d)" % (i, n, n))
+            raise IndexError(f"Index {i} out of range [-{n}, {n})")
         if i < 0:
             i += n
         return i
@@ -662,7 +661,7 @@ class Blocks:
                 parts.append("...")
             else:
                 parts.append(repr(b))
-        return "Block([%s])" % (", ".join(parts),)
+        return "Block([{}])".format(", ".join(parts))
 
 
 class _Overrun:
@@ -842,7 +841,7 @@ class ConjectureData:
 
     def __assert_not_frozen(self, name):
         if self.frozen:
-            raise Frozen("Cannot call %s on frozen ConjectureData" % (name,))
+            raise Frozen(f"Cannot call {name} on frozen ConjectureData")
 
     def note(self, value):
         self.__assert_not_frozen("note")
@@ -883,15 +882,11 @@ class ConjectureData:
             if not at_top_level:
                 return strategy.do_draw(self)
             else:
+                strategy.validate()
                 try:
-                    strategy.validate()
-                    try:
-                        return strategy.do_draw(self)
-                    finally:
-                        self.draw_times.append(time.perf_counter() - start_time)
-                except BaseException as e:
-                    mark_for_escalation(e)
-                    raise
+                    return strategy.do_draw(self)
+                finally:
+                    self.draw_times.append(time.perf_counter() - start_time)
         finally:
             self.stop_example()
 
